package com.acme;

import com.sun.grizzly.tcp.Adapter;
import com.sun.grizzly.tcp.CompletionHandler;
import com.sun.grizzly.tcp.Request;
import com.sun.grizzly.tcp.Response;
import java.io.IOException;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;


public class MyAdapter implements Adapter
{
    private static final int MAX_REQUEST = 500;
    
    private ThreadPoolExecutor          executor;
    private CompletionHandler<Response> completion;
    private AtomicLong                  adapterChooser;
    private AtomicLong                  rejectedChooser;


    public MyAdapter()
        throws IOException
    {
        executor = new ThreadPoolExecutor( 0, MAX_REQUEST, 5*60, TimeUnit.SECONDS, new SynchronousQueue<Runnable>() );
        completion = new MyCompletionHandler();
        adapterChooser = new AtomicLong( -1 );
        rejectedChooser = new AtomicLong( -1 );
    }

    @Override
    public void service( Request req, Response res )
        throws Exception
    {
        res.suspend( Long.MAX_VALUE, res, completion );
        try
        {
            executor.execute( new MyExecutor(req,res,chooseAdapter()) );
        }
        catch( RejectedExecutionException ex )
        {
            res.setStatus( 509 );
            res.setMessage( "Too many request" );
            res.resume();
        }
    }

    @Override
    public void afterService( Request req, Response res )
        throws Exception
    {
        // Nothing !
    }

    private Adapter chooseAdapter()
    {
        if( (adapterChooser.incrementAndGet() % 2) == 0)
        {
            return new MySimpleAdapter( (rejectedChooser.incrementAndGet() % 5) == 0 );
        }
        else
        {
            return new MyGrizzlyAdapter( (rejectedChooser.incrementAndGet() % 5) == 0 );
        }
    }

}
