package com.acme.test;

import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import com.acme.MyAdapter;
import com.sun.grizzly.http.SelectorThread;
import com.sun.grizzly.standalone.StaticStreamAlgorithm;
import com.sun.grizzly.tcp.Adapter;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import org.apache.commons.httpclient.HttpConnectionManager;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

/**
 *
 */
public class TestSuspend
{
    private static final int NB_CLIENT = 100;
    private static final int NB_PASS = 10;
    
    int             port = 35000;
    SelectorThread  selector;
    Adapter         adapter;
    String          url;
    AtomicInteger   succes;
    AtomicInteger   busy;
    HttpConnectionManager   manager;

    static
    {
        //Logger.getLogger( "org.apache.commons.httpclient" ).setLevel( Level.OFF );
    }

    @Before
    public void before()
        throws Exception
    {
        // Create the adapter
        adapter = new MyAdapter( );

        // Create the selector
        selector = new SelectorThread();
        selector.setPort( port );
        selector.setEnableAsyncExecution( false );
        selector.setAdapter( adapter );
        selector.setAlgorithmClassName( StaticStreamAlgorithm.class.getName() );
        selector.setDisplayConfiguration( true );
        selector.setMaxThreads( 5 );
        //selector.setKeepAliveTimeoutInSeconds( 5 );
        //selector.setMaxKeepAliveRequests( 2 );

        // Start
        selector.listen();

        // Others
        succes = new AtomicInteger(0);
        busy = new AtomicInteger(0);
        url = "http://localhost:"+port+"/ping";
        manager = new MultiThreadedHttpConnectionManager();
        manager.getParams().setDefaultMaxConnectionsPerHost( NB_CLIENT*2 );
        manager.getParams().setMaxTotalConnections( NB_CLIENT*2 );
        
    }

    @After
    public void after()
    {
        selector.stopEndpoint();
    }

    
    @Test
    public void testManyRequestsInParallel()
        throws Exception
    {
        ThreadPoolExecutor  executor;

        executor = new ThreadPoolExecutor( NB_CLIENT, NB_CLIENT, 5*60, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>() );

        // Start the clients
        for( int i=0; i<NB_CLIENT; i++ )
        {
            executor.execute( new Caller(url,NB_PASS,succes,busy,manager) );
        }

        // Wait for the end
        try
		{
			executor.shutdown();
			executor.awaitTermination( 15, TimeUnit.MINUTES );
		}
		catch (InterruptedException e)
		{
			fail( "Interrupted during requesters execution." );
		}

        // Show results
        System.out.println();
        System.out.println( "** Success: "+succes.get() );
        System.out.println( "** Busy   : "+busy.get() );
        System.out.println( "** Total  : "+(NB_CLIENT*NB_PASS) );
        System.out.println();

        assertEquals( (NB_CLIENT*NB_PASS), succes.get()+busy.get() );
    }

}
